from readvalid import ReadFiles


class SetOperations(ReadFiles):

    def __init__(self, file1, file2):
        self.hm1 = self.read_file(file1)
        self.hm2 = self.read_file(file2)

    def union(self):
        hm = {**self.hm1, **self.hm2}
        return hm

    def intersection(self):
        hm = {}
        if len(self.hm1) < len(self.hm2):
            s, b = self.hm1, self.hm2
            for i in b:
                if i in s:
                    hm[i] = True
            return hm
        elif len(self.hm1) > len(self.hm2):
            s, b = self.hm1, self.hm2
            for i in b:
                if i in s:
                    hm[i] = True
            return hm

    def minus(self):
        hm = {}
        if len(self.hm1) < len(self.hm2):
            s, b = self.hm1, self.hm2
            for i in s:
                if i not in b:
                    hm[i] = True
            return hm
        elif len(self.hm1) > len(self.hm2):
            s, b = self.hm1, self.hm2
            for i in s:
                if i not in b:
                    hm[i] = True
            return hm



from readvalid import *
from operations import *
from parsearg import *
import sys, time


class ResultFile:
    def __init__(self, file3, func):
        self.file3 = file3
        self.func = func

    def return_file(self):
        with open(self.file3, 'w') as file:
            definition = self.func()
            for i in definition:
                file.write(i)
        with open(self.file3, 'r') as file:
            line = file.readlines()
            return line


if __name__ == "__main__":
    try:
        args = Args(file1, file2).arg_parse()
        if len(sys.argv) == 4:
            start_time = time.time()
            operations = SetOperations(sys.argv[1], sys.argv[2])
            functions = [operations.union, operations.intersection, operations.minus]
            if sys.argv[0] == "Union.py":
                h3 = ResultFile(sys.argv[3], functions[0]).return_file()
            elif sys.argv[0] == "Intersection.py":
                h3 = ResultFile(sys.argv[3], functions[1]).return_file()
            elif sys.argv[0] == "Minus.py":
                h3 = ResultFile(sys.argv[3], functions[2]).return_file()
            end_time = time.time()
            elapsed_time = end_time - start_time
            print(f"Output: {sys.argv[1]}: {len(operations.hm1)} emails, {sys.argv[2]}: {len(operations.hm2)} emails, {sys.argv[3]}: {len(h3)} emails; Time taken: {int(elapsed_time)} seconds")
        else:
            raise IndexError
    except IndexError:
        print("Arguments Validation Failed", "\n", "use syntax like: '<set-operation>.py L1.txt L2.txt R.txt'")


from parsearg import *


class ReadFiles:

    def email_validation(self, email):
        if '@' and '.' in email:
            email_list = email.split("@")
            username, domain_name = email_list[0], email_list[1]
            if 0 < len(username) < 255 and 0 < len(domain_name) < 55:
                if email.index("@") > 0 and email.index('.') < len(email) - 1:
                    top_level_domain = email.split('.')[-1]
                    if 2 <= len(top_level_domain) <= 6:
                        return email

    def read_file(self, file):
        Args.file_validation(self, file, file1, file2)
        with open(file, "r") as file:
            line = file.readlines()
            hm = {}
            for i in line:
                try:
                    if self.email_validation(i):
                        hm[i] = True
                    else:
                        raise SyntaxError
                except SyntaxError:
                    print("Email Format is invalid and removing the unsupported formatted emails")
            return hm



import argparse, sys, os


file1 = "D:/Python/arg-parse-class/class/L1.txt"
file2 = "D:/Python/arg-parse-class/class/L2.txt"


class Args:
    def __init__(self, file1, file2):
        self.file1 = file1
        self.file2 = file2

    def arg_parse(self):
        parser = argparse.ArgumentParser(usage="python %(prog)s L1.txt L2.txt R.txt",
                                         epilog="We have to select the type of set-operation before executing the program",
                                         description="Performing multiple Set Operations based on the User Input")
        parser.add_argument("L1.txt", type=self.file_validation('L1.txt', self.file1, self.file2),
                            help="Name of the 1st file to be processed", default="L1.txt")
        parser.add_argument("L2.txt", type=self.file_validation('L2.txt', self.file1, self.file2),
                            help="Name of the 2nd file to be processed", default="L2.txt")
        parser.add_argument("R.txt", help="Name of the file to be created as an end result", default="R.txt")
        args = parser.parse_args()
        return args

    def file_validation(self, file, file1, file2):
        file_exists = os.path.isfile(file)
        try:
            if file_exists and file in file1 or file2:
                file_not_empty = os.path.getsize(file)
                if file_not_empty:
                    print(f"{file} exists")
                else:
                    print(f"{file} of the input files must be empty, make sure to have the required data")
                    sys.exit(1)
            else:
                raise FileNotFoundError
        except FileNotFoundError:
            print(file, "is not found or could be both files not found or wrong file/s is/are specified \n Make sure "
                        f"to have the required input files present which are {file1}, {file2}")
            sys.exit(1)


if __name__ == "__main__":
    try:
        Args.arg_parse()
        sys.exit(0)
    except Exception as e1:
        print('COMMAND FAILURE: {0}'.format(str(e1)))
        import traceback
        traceback.print_exc()
        sys.exit(1)
        



from setuptools import setup

setup(name='oops_package',
      version='0.0',
      description='Set Operations(Union, Intersection,Minus)',
      packages=['oops_package'],
      author_email='raghavshetty319@gmail.com',
      zip_safe=False)
